@@ -88,7 +88,7 @@ class Consul(object):
         By default the datacenter of the host is used.
         """
 
-        # TODO: Event, Status
+        # TODO: Status
 
         self.http = self.connect(host, port, scheme)
         self.token = token
@@ -97,6 +97,7 @@ class Consul(object):
         assert consistency in ('default', 'consistent', 'stale')
         self.consistency = consistency
 
+        self.event = Consul.Event(self)
         self.kv = Consul.KV(self)
         self.agent = Consul.Agent(self)
         self.catalog = Consul.Catalog(self)
@@ -104,6 +105,104 @@ class Consul(object):
         self.session = Consul.Session(self)
         self.acl = Consul.ACL(self)
 
+    class Event(object):
+        """
+        The event command provides a mechanism to fire a custom user event to
+        an entire datacenter. These events are opaque to Consul, but they can
+        be used to build scripting infrastructure to do automated deploys,
+        restart services, or perform any other orchestration action.
+
+        Unlike most Consul data, which is replicated using consensus, event
+        data is purely peer-to-peer over gossip.
+
+        This means it is not persisted and does not have a total ordering. In
+        practice, this means you cannot rely on the order of message delivery.
+        An advantage however is that events can still be used even in the
+        absence of server nodes or during an outage."""
+        def __init__(self, agent):
+            self.agent = agent
+
+        def fire(
+                self,
+                name,
+                body="",
+                node=None,
+                service=None,
+                tag=None):
+            """
+            Sends an event to Consul's gossip protocol.
+
+            *name* is the Consul-opaque name of the event. This can be filtered
+            on in calls to list, below
+
+            *body* is the Consul-opaque body to be delivered with the event.
+             From the Consul documentation:
+                The underlying gossip also sets limits on the size of a user
+                event message. It is hard to give an exact number, as it
+                depends on various parameters of the event, but the payload
+                should be kept very small (< 100 bytes). Specifying too large
+                of an event will return an error.
+
+            *node*, *service*, and *tag* are regular expressions which remote
+            agents will filter against to determine if they should store the
+            event
+            """
+            assert not name.startswith('/')
+            params = {}
+            if node is not None:
+                params['node'] = node
+            if service is not None:
+                params['service'] = service
+            if tag is not None:
+                params['tag'] = tag
+
+            return self.agent.http.put(
+                callback(is_json=True),
+                '/v1/event/fire/%s' % name, params=params, data=body)
+
+        def list(
+                self,
+                name=None):
+            """
+            Returns a tuple of (*index*, *events*)
+                Note: Since Consul's event protocol uses gossip, there is no
+                ordering, and instead index maps to the newest event that
+                matches the query.
+
+            *name* is the type of events to list, if None, lists all available.
+
+            Consul agents only buffer the most recent entries. The current
+            buffer size is 256, but this value could change in the future.
+
+            Each *event* looks like this::
+
+                {
+                      {
+                        "ID": "b54fe110-7af5-cafc-d1fb-afc8ba432b1c",
+                        "Name": "deploy",
+                        "Payload": "1609030",
+                        "NodeFilter": "",
+                        "ServiceFilter": "",
+                        "TagFilter": "",
+                        "Version": 1,
+                        "LTime": 19
+                      },
+                }
+            """
+            params = {}
+            if name is not None:
+                params['name'] = name
+
+            def callback(response):
+                data = json.loads(response.body)
+                for item in data:
+                    if item.get('Payload') is not None:
+                        item['Payload'] = base64.b64decode(item['Payload'])
+                return response.headers['X-Consul-Index'], data
+
+            return self.agent.http.get(
+                callback, '/v1/event/list', params=params)
+
     class KV(object):
         """
         The KV endpoint is used to expose a simple key/value store. This can be
@@ -121,6 +220,8 @@ class Consul(object):
                 wait=None,
                 token=None,
                 consistency=None,
+                keys=False,
+                separator=None,
                 dc=None):
             """
             Returns a tuple of (*index*, *value[s]*)
@@ -132,6 +233,10 @@ class Consul(object):
 
             *token* is an optional `ACL token`_ to apply to this request.
 
+            *keys* is a boolean which, if True, says to return a flat list of
+            keys without values or other metadata. *separator* can be used
+            with *keys* to list keys only up to a given separator character.
+
             *dc* is the optional datacenter that you wish to communicate with.
             If None is provided, defaults to the agent's datacenter.
 
@@ -169,6 +274,10 @@ class Consul(object):
             dc = dc or self.agent.dc
             if dc:
                 params['dc'] = dc
+            if keys:
+                params['keys'] = True
+            if separator:
+                params['separator'] = separator
             consistency = consistency or self.agent.consistency
             if consistency in ('consistent', 'stale'):
                 params[consistency] = '1'
@@ -180,11 +289,12 @@ class Consul(object):
                     data = None
                 else:
                     data = json.loads(response.body)
-                    for item in data:
-                        if item.get('Value') is not None:
-                            item['Value'] = base64.b64decode(item['Value'])
-                    if not recurse:
-                        data = data[0]
+                    if not keys:
+                        for item in data:
+                            if item.get('Value') is not None:
+                                item['Value'] = base64.b64decode(item['Value'])
+                        if not recurse:
+                            data = data[0]
                 return response.headers['X-Consul-Index'], data
 
             return self.agent.http.get(
